#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "StackFunc.h"
#include "AkelDLL.h"


//Include stack functions
#define StackInsertAfter
#define StackInsertBefore
#define StackInsertIndex
#define StackDelete
#define StackClear
#include "StackFunc.h"

//Defines
#define BUFFER_SIZE      1024

typedef struct _FONTNAMEA {
  struct _FONTNAMEA *next;
  struct _FONTNAMEA *prev;
  char szFont[MAX_PATH];
} FONTNAMEA;

typedef struct _FONTNAMEW {
  struct _FONTNAMEW *next;
  struct _FONTNAMEW *prev;
  wchar_t wszFont[MAX_PATH];
} FONTNAMEW;

//Functions prototypes
char* GetFileExtA(char *pFile);
wchar_t* GetFileExtW(wchar_t *wpFile);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
HWND hMainWnd;
BOOL bOldWindows;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
char szFontsDir[MAX_PATH];
wchar_t wszFontsDir[MAX_PATH];
char szBuffer[BUFFER_SIZE];
wchar_t wszBuffer[BUFFER_SIZE];
HSTACK hFontsStack={0};
BOOL bFontsAdded=FALSE;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="Fonts";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  //Is plugin already loaded?
  if (bInitMain)
  {
    UninitMain();
  }
  else
  {
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

char* GetFileExtA(char *pFile)
{
  int i;

  for (i=lstrlenA(pFile) - 1; i >= 0; --i)
  {
    if (pFile[i] == '.') return (pFile + i + 1);
    else if (pFile[i] == '\\') break;
  }
  return "";
}

wchar_t* GetFileExtW(wchar_t *wpFile)
{
  int i;

  for (i=lstrlenW(wpFile) - 1; i >= 0; --i)
  {
    if (wpFile[i] == '.') return (wpFile + i + 1);
    else if (wpFile[i] == '\\') break;
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hMainWnd=pd->hMainWnd;
  bOldWindows=pd->bOldWindows;

  if (bOldWindows)
    wsprintfA(szFontsDir, "%s\\AkelFiles\\Plugs\\Fonts", (char *)pd->pAkelDir);
  else
    wsprintfW(wszFontsDir, L"%s\\AkelFiles\\Plugs\\Fonts", (wchar_t *)pd->pAkelDir);
}

void InitMain()
{
  bInitMain=TRUE;

  if (bOldWindows)
  {
    FONTNAMEA *fn;
    WIN32_FIND_DATAA wfd;
    HANDLE hFind;
    char szFindFiles[MAX_PATH];
    char *pExt;

    wsprintfA(szFindFiles, "%s\\*.*", szFontsDir);

    if ((hFind=FindFirstFileA(szFindFiles, &wfd)) != INVALID_HANDLE_VALUE)
    {
      do
      {
        if (wfd.cFileName[0] == '.' && (wfd.cFileName[1] == '\0' || (wfd.cFileName[1] == '.' && wfd.cFileName[2] == '\0'))) continue;

        if (pExt=GetFileExtA(wfd.cFileName))
        {
          if (!lstrcmpiA(pExt, "fon") ||
              !lstrcmpiA(pExt, "fnt") ||
              !lstrcmpiA(pExt, "ttf") ||
              !lstrcmpiA(pExt, "ttc") ||
              !lstrcmpiA(pExt, "fot") ||
              !lstrcmpiA(pExt, "otf") ||
              !lstrcmpiA(pExt, "mmm") ||
              !lstrcmpiA(pExt, "pfb") ||
              !lstrcmpiA(pExt, "pfm"))
          {
            wsprintfA(szBuffer, "%s\\%s", szFontsDir, wfd.cFileName);
            AddFontResourceA(szBuffer);
            bFontsAdded=TRUE;

            if (!StackInsertIndex((stack **)&hFontsStack.first, (stack **)&hFontsStack.last, (stack **)&fn, -1, sizeof(FONTNAMEA)))
            {
              lstrcpynA(fn->szFont, szBuffer, MAX_PATH);
            }
          }
        }
      }
      while (FindNextFileA(hFind, &wfd));

      FindClose(hFind);
    }
  }
  else
  {
    FONTNAMEW *fn;
    WIN32_FIND_DATAW wfd;
    HANDLE hFind;
    wchar_t wszFindFiles[MAX_PATH];
    wchar_t *wpExt;

    wsprintfW(wszFindFiles, L"%s\\*.*", wszFontsDir);

    if ((hFind=FindFirstFileW(wszFindFiles, &wfd)) != INVALID_HANDLE_VALUE)
    {
      do
      {
        if (wfd.cFileName[0] == '.' && (wfd.cFileName[1] == '\0' || (wfd.cFileName[1] == '.' && wfd.cFileName[2] == '\0'))) continue;

        if (wpExt=GetFileExtW(wfd.cFileName))
        {
          if (!lstrcmpiW(wpExt, L"fon") ||
              !lstrcmpiW(wpExt, L"fnt") ||
              !lstrcmpiW(wpExt, L"ttf") ||
              !lstrcmpiW(wpExt, L"ttc") ||
              !lstrcmpiW(wpExt, L"fot") ||
              !lstrcmpiW(wpExt, L"otf") ||
              !lstrcmpiW(wpExt, L"mmm") ||
              !lstrcmpiW(wpExt, L"pfb") ||
              !lstrcmpiW(wpExt, L"pfm"))
          {
            wsprintfW(wszBuffer, L"%s\\%s", wszFontsDir, wfd.cFileName);
            AddFontResourceW(wszBuffer);
            bFontsAdded=TRUE;

            if (!StackInsertIndex((stack **)&hFontsStack.first, (stack **)&hFontsStack.last, (stack **)&fn, -1, sizeof(FONTNAMEW)))
            {
              lstrcpynW(fn->wszFont, wszBuffer, MAX_PATH);
            }
          }
        }
      }
      while (FindNextFileW(hFind, &wfd));

      FindClose(hFind);
    }
  }

  if (bFontsAdded)
  {
    //SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
  }
}

void UninitMain()
{
  bInitMain=FALSE;

  if (bOldWindows)
  {
    FONTNAMEA *lpElement=(FONTNAMEA *)hFontsStack.first;

    while (lpElement)
    {
      RemoveFontResourceA(lpElement->szFont);

      lpElement=lpElement->next;
    }
  }
  else
  {
    FONTNAMEW *lpElement=(FONTNAMEW *)hFontsStack.first;

    while (lpElement)
    {
      RemoveFontResourceW(lpElement->wszFont);

      lpElement=lpElement->next;
    }
  }
  StackClear((stack **)&hFontsStack.first, (stack **)&hFontsStack.last);

  if (bFontsAdded)
  {
    //SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
    bFontsAdded=FALSE;
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
